# titanium-lib

Simple collection of tools for developing Titanium mobile apps. It consists of
a set of small modules that work together to provide a set of reusable
functions that remove the need for rewriting basic things in each app. This
library grows from my own development needs and hence may not be all that
useful to anybody else.

## Available modules

### audio

Minimal code for playing sounds. Zero error checking.

- `loadSound(name, path)` Preloads the sound file
- `playSound(name)` Plays a preloaded sound

### cache

Data caching system. Functionality is still being developed.

- `set(key, value, lifetime)`
- `get(key, evenIfExpired, fullObject)`
- `refresh(key)`
- `isCached(key)`
- `isExpired(key)`
- `setKeyPrefix(keyPrefix)`

### config

Configuration handler with the possiblity to have different settings based on
a combination of platform (iOS/Android), form factor (tablet/handheld),
current devide orientation (landscape, portrait).

- `load(config, preParse)`
- `get(subArea, key, fallback)`

### debug

Debugging tools. This is pretty project-specific and has to be retooled.

### lib

Convenience wrapper used to load all the actual libraries.

- `load(libs, incPath)`
- `enableLogging()`
- `loggingIgnorePrefixes(prefixes)`
- `log = function(msg, prefix, subPrefix)`

### net

Network tools

- `isOnline()`
- `getJson(url, callback, errorCallback, returnString)`

### phpjs

About to be removed. Collection of functions from phpjs, used in other modules.

### profile

Simple time profiling tools.

- `start(label)`
- `step(subLabel, label)`
- `report(label)`

### rotate

Rotation handling

- `enable()`
- `disable()`
- `on(callback, notNow)`
- `getOrientation()`
- `isAllowedOrientation(orientation)`
- `deny(orientations)`
- `allowPortrait()`
- `denyPortrait()`
- `allowLandscape()`
- `denyLandscape()`
- `allowFace()`
- `denyFace()`

### system

Basic os detection and app handling

- `load()`
- `firstRun(fn)`

### tools

Various misc. tools

- `log(msg)`
- `setBool(name, data)`
- `getBool(name, fallback)`
- `setString(name, data)`
- `getString(name, fallback)`
- `setInt(name, data)`
- `getInt(name, fallback)`
- `setObject(name, data)`
- `getObject(name, fallback)`

### UI

Interface tools. Still very app-specific and half-baked.

### window

Manages multiple windows.

- `load(windowList, firstWindow, incPath)`
- `open(windowName, customOptions)`
- `close(windowName)`
- `closeLast()`
- `closeAll()`
- `setBackground(color)`

### xml

System to work with remote and local XML files as data sources

- `load(options)`

## TODO

- Remove dependency on phpjs functions, remove phpjs module
- Move property get/set functions from tools to new separate module
- Manage dependencies between modules better
- Remove cross-dependencies as much as possible
- lib.load: Figure out include path automatically
- Unify variable and function names. `load` is used with different meanings.
