// Plans: Add list of cached objects, add function to remove expired objects.

var cache = function() {
	var self = {
		keyPrefix: 'default',
		defaultLifetime: 60*60*24
	};

	self.set = function(key, value, lifetime) {
		lib.tools.setObject(self.generatePropertyKey(key), {
			value: value,
			lifetime: lifetime,
			expires: lifetime + new Date()
		});
	};
	
	self.get = function(key, evenIfExpired, fullObject) {
		var data = lib.tools.getObject(self.generatePropertyKey(key), false);
		if(typeof(data)=='object' && typeof(data.value)!='undefined') {
			var now = +new Date;
			if(evenIfExpired || now<data.expires) {
				if(fullObject) {
					return data;
				} else {
					return data.value;
				}
			}
		}
		return false;
	};
	
	self.refresh = function(key) {
		var data = self.get(key);
		self.set(key, data);
	};

	self.isCached = function(key) {
		var data = self.get(key, true, true);
		return data!==false;
	};
	
	self.isExpired = function(key) {
		var data = self.get(key, true, true);
		if(data==false) {
			return true;
		}
		var now = +new Date;
		return now<=data.expires;
	};
	
	self.generatePropertyKey = function(key) {
		var propertyKey = 'lib.cache-'+self.keyPrefix+'-'+key;
		return propertyKey;
	};
	
	self.setKeyPrefix = function(keyPrefix) {
		self.keyPrefix = keyPrefix;
	};
	
	return self;
};

module.exports = cache();
