// requires lib.tools

// Note: For rotation with this lib to work correctly, the app must be set up with ONLY portrait rotation.

var rotate = function() {
	var self = {
		enabled: false,
		changeWaitPeriod: 750, // 350
		changeTimeout: null,
		orientation: Ti.UI.PORTRAIT,
		allowedOrientations: [
			Ti.UI.PORTRAIT,
			Ti.UI.UPSIDE_PORTRAIT,
			Ti.UI.LANDSCAPE_RIGHT,
			Ti.UI.LANDSCAPE_LEFT
		],
		portraitOrientations: [
			Ti.UI.PORTRAIT,
			Ti.UI.UPSIDE_PORTRAIT
		],
		landscapeOrientations: [
			Ti.UI.LANDSCAPE_RIGHT,
			Ti.UI.LANDSCAPE_LEFT
		],
		faceOrientations: [
			Ti.UI.FACE_UP,
			Ti.UI.FACE_DOWN
		]
	};

	self.orientationchange = function() {
		var newOrientation = Titanium.Gesture.orientation;
		var doChange = self.enabled;
		if(!self.isAllowedOrientation(newOrientation)) {
			if(self.isAllowedOrientation(self.orientation)) {
				newOrientation = self.orientation;
			} else {
				newOrientation = self.allowedOrientations[0];
			}
			// TODO: Maybe find a way to find the closest matching orientation?
		}
		if(doChange) {
			clearTimeout(self.changeTimeout);
			self.changeTimeout = setTimeout(function() {
				self.orientation = newOrientation;
				Titanium.UI.orientation = self.orientation;
				Ti.fireEvent('libOrientationChange', {orientation: self.orientation});
				self.log('libOrientationChange '+self.orientation, 'orientationchange');
			}, self.changeWaitPeriod);
		}
	};

	self.isAllowedOrientation = function(orientation) {
		for(var o in self.allowedOrientations) {
			if(self.allowedOrientations[o]==orientation) {
				return true;
			}
		}
		return false;
	}
	
	self.enable = function() {
		self.log('enable');
		self.enabled = true;
		Titanium.Gesture.addEventListener('orientationchange', self.orientationchange);
		self.orientationchange();
	};

	self.disable = function() {
		self.log('disable');
		self.enabled = false;
		Titanium.Gesture.removeEventListener('orientationchange', self.orientationchange);
	};

	self.on = function(callback, notNow) {
		var on = function(orientation) {
			if(typeof(lib.window.appWindow)=='object') {
				callback(orientation);
			} else {
				callback(orientation);
			}
		};
		Ti.addEventListener('libOrientationChange', function() {
			on(self.orientation);
		});
		if(!notNow) {
			on(self.orientation);
		}
	};
	
	self.getOrientation = function() {
		// TODO: How to handle FACE_DOWN, FACE_UP?
		return (self.orientation==Titanium.UI.LANDSCAPE_LEFT || self.orientation==Titanium.UI.LANDSCAPE_RIGHT) ? 'landscape' : 'portrait';
	};

	// self.setOrientation = function(newOrientation) {
	// 	self.orientationchange({}, newOrientation);
	// };

	self.allow = function(orientations) {
		for(var o in orientations) {
			self.allowedOrientations.push(orientations[o]);
			self.log(orientations[o], 'allow');
		}
		self.orientationchange();
	};

	self.deny = function(orientations) {
		for(var o in orientations) {
			for(var i in self.allowedOrientations) {
				if(self.allowedOrientations[i]==orientations[o]) {
					self.allowedOrientations.splice(i, 1);
					self.log(orientations[o], 'deny');
				}
			}
		}
		self.orientationchange();
	};

	self.allowPortrait = function() {
		self.allow(self.portraitOrientations);
	};

	self.denyPortrait = function() {
		self.deny(self.portraitOrientations);
	};

	self.allowLandscape = function() {
		self.allow(self.landscapeOrientations);
	};

	self.denyLandscape = function() {
		self.deny(self.landscapeOrientations);
	};

	self.allowFace = function() {
		self.allow(self.faceOrientations);
	};

	self.denyFace = function() {
		self.deny(self.faceOrientations);
	};

	return self;
};

module.exports = rotate();