var xml = function() {
	var self = {
	};

	self.load = function(customOptions) {
		var options = {
			url: null,
			format: 'rss',
			rssItemFields: [
				{key: 'title'},
				{key: 'link'},
				{key: 'description'},
				{key: 'playingtime'}, // TODO: Move to video things somehow
			]
			// success: function
			// success2: function
			// error: function
			// error2: function
			// update: function
			// update2: function
		};
		for(o in customOptions) {
			options[o] = customOptions[o];
		}
		if(self.isRemoteUrl(options.url)) {
			self.loadUrl(options);
		} else {
			self.loadLocal(options);
		}
	};

	self.isRemoteUrl = function(url) {
		var exp = new RegExp('^(https?:\/\/)', 'i');
		return exp.test(url);
	};

	self.loadLocal = function(options) {
		if(typeof(options.url)=='string') {
			var xmlFile = Ti.Filesystem.getFile(options.url);
			if(xmlFile.exists()) {
				var xmlString = xmlFile.read().toString();
				var data = self.parse(xmlString, options);
				if(data!=false) {
					self.callback(options.success, data);
					self.callback(options.success2, data);
				}
				return true;
			}
		}
		self.callback(options.error);
		return false;
	};

	self.loadUrl = function(options) {
		var xhr = Titanium.Network.createHTTPClient();
		if(typeof(options.update)=='function') {
			xhr.ondatastream = options.update;
		}
		xhr.onload = function() {
			var data = self.parse(this.responseData.toString(), options);
			if(data===false) {
				self.callback(options.error);
			} else {
				self.callback(options.success, data);
				self.callback(options.success2, data);
			}
		};
		if(typeof(options.error)=='function') {
			xhr.onerror = options.error;
		}
		xhr.open('GET', options.url);
		xhr.send();
		self.abort = function() {
			xhr.abort();
		}
	};

	self.parse = function(xmlString, options) {
		var data;
		try {
			var xml = Titanium.XML.parseString(xmlString);
			switch(options.format) {
				case 'rss':
				default:
					data = self.parseRSS(xml, options);
					break;
			}
		} catch(e) {
			data = false;
			self.log('parse: caught error:');
			self.log(e);
		}
		return data;
	};

	self.parseRSS = function(xml, options) {
		var data = [];
		var items = xml.documentElement.getElementsByTagName('item');
		for (var i = 0; i < items.length; i++) {
			var item = items.item(i);
			var itemData = {};
			for(var f in options.rssItemFields) {
				itemData[options.rssItemFields[f].key] = self.getRssTagText(item, options.rssItemFields[f].key, typeof(options.rssItemFields[f].ns)!='undefined'?options.rssItemFields[f].ns:false);
			}
			data.push(itemData);
		}
		return data;
	};

	self.getRssTagText = function(tag, key, ns) {
		var text;
		try {
			if(ns) {
				text = tag.getElementsByTagNameNS(ns, key).item(0).text;
			} else {
				text = tag.getElementsByTagName(key).item(0).text;
			}
		} catch(e) {
			text = '';
		}
		return text;
	};

	self.callback = function(fn, data) {
		if(typeof(fn)=='function') {
			fn(data);
		}
	}

	return self;
}

module.exports = xml();
